import UIKit
import RxSwift

let disposeBag = DisposeBag()

func test<T>(_ subject: T, count: Int) where T: SubjectType, T.SubjectObserverType.E == Int {
    var subscriptionResults = [String]()
    
    for i in (0...count) {
        subscriptionResults.append("--")
        
        subject
            .subscribe(
                onNext: { value in
                    subscriptionResults[i] += " \(value) --"
                },
                onCompleted: {
                    print("\(subject.self)Subscription\(i): \(subscriptionResults[i])>")
                })
            .disposed(by: disposeBag)
        
        subject.asObserver().onNext(i)
    }
    
    subject.asObserver().onCompleted()
}

// PublishSubject

let publishSubject = PublishSubject<Int>()

test(publishSubject, count: 2)
print("\n********************\n")

// BehaviorSubject

let behaviorSubject = BehaviorSubject<Int>(value: 0)

test(behaviorSubject, count: 5)
print("\n********************\n")

// ReplaySubject

let replaySubject = ReplaySubject<Int>.create(bufferSize: 2)

test(replaySubject, count: 7)
print("\n********************\n")

// AsyncSubject

let asyncSubject = AsyncSubject<Int>()

test(asyncSubject, count: 3)
print("\n********************\n")


// Relays

import RxCocoa

// PublishRelay

let publishRelay = PublishRelay<Int>()

publishRelay.subscribe(onNext: { value in
    print(value)
})

publishRelay.accept(1)
publishRelay.accept(2)

print("\n********************\n")

// BehaviorRelay

let behaviorRelay = BehaviorRelay<Int>(value: 0)

behaviorRelay.subscribe(onNext: { value in
    print(value)
})

behaviorRelay.accept(1)
behaviorRelay.accept(2)
